#include "stm32f10x.h"
#include "stm32f10x_tim.h"
#include "stm32f10x_rcc.h"
#include "delay.h"

volatile uint16_t mult=25;  // external quarz 25mhz

// **
void delay_init(void)
{
	RCC_ClocksTypeDef RCC_Clocks;

    RCC_GetClocksFreq(&RCC_Clocks);
    mult=RCC_Clocks.SYSCLK_Frequency/1000000;

}

// **
void delay_us(uint32_t time_us)
{

	/* The RCC feeds the Cortex System Timer (SysTick) external clock with the AHB clock
	(HCLK) divided by 8. The SysTick can work either with this clock or with the Cortex clock
	(HCLK), configurable in the SysTick Control and Status Register. */

  SysTick->LOAD  =  (mult * time_us / 8)-1;						   // maximum 24 bit after mult
  SysTick->VAL   = 0;                                          // Load the SysTick Counter Value
  SysTick->CTRL  |= SysTick_CTRL_ENABLE_Msk;                   // Enable SysTick Timer

  do
  {
  } while ((SysTick->CTRL & SysTick_CTRL_COUNTFLAG)==0);
  SysTick->CTRL  &= ~SysTick_CTRL_ENABLE_Msk;                  // Disable SysTick Timer
  SysTick->VAL   = 0;                                          // Load the SysTick Counter Value
}


// **
void delay_ms(uint16_t time_ms)
{
	  while (time_ms>0)
	  {
	    delay_us(1000);
	    time_ms--;
	  }
}

