
#ifndef __DS18x20_H
#define __DS18x20_H

/* -------------------------------------------------------------------------- */
/*                         DS1820 Timing Parameters                           */
/* -------------------------------------------------------------------------- */
#define DS1820_RST_PULSE       480               /* master reset pulse time in [us] */
#define DS1820_MSTR_BITSTART   2                 /* delay time for bit start by master */
#define DS1820_PRESENCE_WAIT   40                /* delay after master reset pulse in [us] */
#define DS1820_PRESENCE_FIN    480               /* dealy after reading of presence pulse [us] */
#define DS1820_BITREAD_DLY     5                 /* bit read delay */
#define DS1820_BITWRITE_DLY    100               /* bit write delay */

/* -------------------------------------------------------------------------- */
/*                            DS1820 Registers                                */
/* -------------------------------------------------------------------------- */

#define DS1820_REG_TEMPLSB    0
#define DS1820_REG_TEMPMSB    1
#define DS1820_REG_CNTREMAIN  6
#define DS1820_REG_CNTPERSEC  7
#define DS1820_SCRPADMEM_LEN  9                     // length of scratchpad memory
#define DS1820_ADDR_LEN       8

/* -------------------------------------------------------------------------- */
/*                            DS1820 Commands                                 */
/* -------------------------------------------------------------------------- */

#define DS1820_CMD_SEARCHROM     0xF0               // recherche des differents DS1820 et de leur numROMs
#define DS1820_CMD_READROM       0x33               // idem que SEARCHROM mais utilise pour 1 seul DS1820
#define DS1820_CMD_MATCHROM      0x55               // permet de communiquer avec un DS1820 en particulier grace a son numROM
#define DS1820_CMD_SKIPROM       0xCC               // permet de communiquer avec tous les DS1820 en meme temps
#define DS1820_CMD_ALARMSEARCH   0xEC               // permet de dialoguer seulement avec les DS1820 qui ont un flag
#define DS1820_CMD_CONVERTTEMP   0x44               // permet de lancer un convertion de la temperature, le resultat sera stocke dans le scratchpad
#define DS1820_CMD_WRITESCRPAD   0x4E               // permet au MBED d'ecrire deux octets dans le scratchpad dont un dans le registre TH et un dans le registre TL
#define DS1820_CMD_READSCRPAD    0xBE               // permet au MBED de lire le scratchpad
#define DS1820_CMD_COPYSCRPAD    0x48               // permet de copier le scratchpad et les registres TH, TL stocke dans EEPROM
#define DS1820_CMD_RECALLEE      0xB8               // rappel les valeur de declenchement de l'alarme
#define DS1820_FAMILY_CODE_DS18B20      0x28
#define DS1820_FAMILY_CODE_DS18S20      0x10




void DS18x20Init(void);
uint8_t First(void);
void FindDevices(void);
float DS1820_GetTemp(void);

#endif
